@@ -1,4 +1,5 @@
-# Copyright 2013-2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,21 +28,21 @@
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Ruturaj Gujar, <ruturaj.gujar23@gmail.com>, 2019
 # - Brandon White, <bjwhite@fnal.gov>, 2019
-# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2019 - 2020
-#
-# PY3K COMPATIBLE
-import json as json_lib
-from six import iteritems
+# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 
+import json as json_lib
+
+from six import iteritems
 from sqlalchemy import String, cast, type_coerce, JSON
 from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy.sql.expression import text
 
 from rucio.common import exception
+from rucio.core.did_meta_plugins.did_meta_plugin_interface import DidMetaPlugin
 from rucio.db.sqla import models
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
-
-from rucio.core.did_meta_plugins.did_meta_plugin_interface import DidMetaPlugin
+from rucio.db.sqla.util import json_implemented
 
 
 class JSONDidMeta(DidMetaPlugin):
@@ -62,7 +63,7 @@ class JSONDidMeta(DidMetaPlugin):
         :param name: The data identifier name.
         :param session: The database session in use.
         """
-        if not self.json_implemented(session):
+        if not json_implemented(session=session):
             raise NotImplementedError
 
         try:
@@ -72,49 +73,44 @@ class JSONDidMeta(DidMetaPlugin):
         except NoResultFound:
             raise exception.DataIdentifierNotFound("No generic metadata found for '%(scope)s:%(name)s'" % locals())
 
-    @transactional_session
-    def set_metadata(self, scope, name, key, value, recursive, session=None):
-        """
-        Add or update the given metadata to the given did
+    def set_metadata(self, scope, name, key, value, recursive=False, session=None):
+        self.set_metadata_bulk(scope=scope, name=name, meta={key: value}, recursive=recursive, session=session)
 
-        :param scope: the scope of the did
-        :param name: the name of the did
-        :param meta: the metadata to be added or updated
-        """
-        if not self.json_implemented(session):
+    @transactional_session
+    def set_metadata_bulk(self, scope, name, meta, recursive=False, session=None):
+        if not json_implemented(session=session):
             raise NotImplementedError
 
-        try:
-            row_did = session.query(models.DataIdentifier).filter_by(scope=scope, name=name).one()
-            row_did_meta = session.query(models.DidMeta).filter_by(scope=scope, name=name).scalar()
-            if row_did_meta is None:
-                # Add metadata column to new table (if not already present)
-                row_did_meta = models.DidMeta(scope=scope, name=name)
-                row_did_meta.save(session=session, flush=False)
-            existing_meta = getattr(row_did_meta, 'meta')
-            # Oracle returns a string instead of a dict
-            if session.bind.dialect.name in ['oracle', 'sqlite'] and existing_meta is not None:
-                existing_meta = json_lib.loads(existing_meta)
+        if session.query(models.DataIdentifier).filter_by(scope=scope, name=name).one_or_none() is None:
+            raise exception.DataIdentifierNotFound("Data identifier '%s:%s' not found" % (scope, name))
 
-            if existing_meta is None:
-                existing_meta = {}
+        row_did_meta = session.query(models.DidMeta).filter_by(scope=scope, name=name).scalar()
+        if row_did_meta is None:
+            # Add metadata column to new table (if not already present)
+            row_did_meta = models.DidMeta(scope=scope, name=name)
+            row_did_meta.save(session=session, flush=False)
 
-            # for k, v in iteritems(meta):
-            #     existing_meta[k] = v
+        existing_meta = {}
+        if hasattr(row_did_meta, 'meta'):
+            if row_did_meta.meta:
+                existing_meta = row_did_meta.meta
 
+        # Oracle returns a string instead of a dict
+        if session.bind.dialect.name in ['oracle', 'sqlite'] and existing_meta:
+            existing_meta = json_lib.loads(existing_meta)
+
+        for key, value in meta.items():
             existing_meta[key] = value
 
-            row_did_meta.meta = None
-            session.flush()
+        row_did_meta.meta = None
+        session.flush()
 
-            # Oracle insert takes a string as input
-            if session.bind.dialect.name in ['oracle', 'sqlite']:
-                existing_meta = json_lib.dumps(existing_meta)
+        # Oracle insert takes a string as input
+        if session.bind.dialect.name in ['oracle', 'sqlite']:
+            existing_meta = json_lib.dumps(existing_meta)
 
-            row_did_meta.meta = existing_meta
-            row_did_meta.save(session=session, flush=True)
-        except NoResultFound:
-            raise exception.DataIdentifierNotFound("Data identifier '%(scope)s:%(name)s' not found" % locals())
+        row_did_meta.meta = existing_meta
+        row_did_meta.save(session=session, flush=True)
 
     @transactional_session
     def delete_metadata(self, scope, name, key, session=None):
@@ -125,7 +121,7 @@ class JSONDidMeta(DidMetaPlugin):
         :param name: the name of the did
         :param key: the key to be deleted
         """
-        if not self.json_implemented(session):
+        if not json_implemented(session=session):
             raise NotImplementedError
 
         try:
@@ -155,7 +151,7 @@ class JSONDidMeta(DidMetaPlugin):
     def list_dids(self, scope, filters, type='collection', ignore_case=False, limit=None,
                   offset=None, long=False, recursive=False, session=None):
         # Currently for sqlite only add, get and delete is implemented.
-        if not self.json_implemented(session):
+        if not json_implemented(session=session):
             raise NotImplementedError
 
         query = session.query(models.DidMeta)
@@ -181,14 +177,9 @@ class JSONDidMeta(DidMetaPlugin):
             for row in query.yield_per(5):
                 yield row.name
 
-    def manages_key(self, key):
-        """
-        Returns whether key is managed by this plugin or not.
-        JSON plugin should be considered a wildcard.
-        :param key: Key of the metadata.
-        :returns (Boolean)
-        """
-        return True
+    @read_session
+    def manages_key(self, key, session=None):
+        return json_implemented(session=session)
 
     def get_plugin_name(self):
         """
@@ -196,22 +187,3 @@ class JSONDidMeta(DidMetaPlugin):
         This can then be used when listing the metadata of did to only provide dids from this plugin.
         """
         return self.plugin_name
-
-    def json_implemented(self, session=None):
-        """
-        Checks if the database on the current server installation can support json fields.
-        Check if did meta json table exists.
-
-        :param session: (Optional) The active session of the database.
-
-        :returns: True, if json is supported, False otherwise.
-        """
-        # if session is None:
-        #     session = se.get_session()
-        if session.bind.dialect.name == 'oracle':
-            oracle_version = int(session.connection().connection.version.split('.')[0])
-            if oracle_version < 12:
-                return False
-        if session.bind.dialect.name == 'sqlite':
-            return False
-        return True
