@@ -1,4 +1,5 @@
-# Copyright 2016-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2016-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,17 +14,15 @@
 # limitations under the License.
 #
 # Authors:
-# - Wen Guan <wguan.icedew@gmail.com>, 2016
-# - Vincent Garonne <vgaronne@gmail.com>, 2016-2018
+# - Wen Guan <wen.guan@cern.ch>, 2016
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2016-2018
 # - Martin Barisits <martin.barisits@cern.ch>, 2017
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Brandon White <bjwhite@fnal.gov>, 2019-2020
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Brandon White <bjwhite@fnal.gov>, 2019
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 
 """
 Conveyor throttler is a daemon to manage rucio internal queue.
@@ -35,13 +34,13 @@ import logging
 import math
 import os
 import socket
-import sys
 import threading
 import time
 import traceback
 
-
-from rucio.common.config import config_get
+import rucio.db.sqla.util
+from rucio.common import exception
+from rucio.common.logging import setup_logging, formatted_logger
 from rucio.common.utils import get_parsed_throttler_mode
 from rucio.core import heartbeat, config as config_core
 from rucio.core.monitor import record_counter, record_gauge
@@ -49,13 +48,6 @@ from rucio.core.request import get_stats_by_activity_direction_state, release_al
 from rucio.core.rse import get_rse, set_rse_transfer_limits, delete_rse_transfer_limits, get_rse_transfer_limits
 from rucio.db.sqla.constants import RequestState
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 graceful_stop = threading.Event()
 
 
@@ -94,7 +86,7 @@ def throttler(once=False, sleep_time=600):
                 continue
 
             logging.info(prepend_str + "Throttler - schedule requests")
-            __schedule_requests()
+            run_once(logger=formatted_logger(logging.log, prepend_str + '%s'))
 
             if once:
                 break
@@ -126,6 +118,11 @@ def run(once=False, sleep_time=600):
     """
     Starts up the conveyer threads.
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise exception.DatabaseException('Database was not updated, daemon won\'t start')
+
     if once:
         logging.info('running throttler one iteration only')
         throttler(once=True, sleep_time=sleep_time)
@@ -211,7 +208,7 @@ def __get_request_stats(all_activities=False, direction='destination'):
     return result_dict
 
 
-def __schedule_requests():
+def run_once(logger=logging.log, session=None):
     """
     Schedule requests
     """
@@ -226,14 +223,14 @@ def __schedule_requests():
                 # dest_rse is not blacklisted for write or src_rse is not blacklisted for read
                 if (direction == 'destination' and availability & 2) or (direction == 'source' and availability & 4):
                     if all_activities:
-                        __release_all_activities(result_dict[rse_id], direction, rse_name, rse_id)
+                        __release_all_activities(result_dict[rse_id], direction, rse_name, rse_id, logger=logger, session=session)
                     else:
-                        __release_per_activity(result_dict[rse_id], direction, rse_name, rse_id)
+                        __release_per_activity(result_dict[rse_id], direction, rse_name, rse_id, logger=logger, session=session)
     except Exception:
-        logging.critical("Failed to schedule requests, error: %s" % (traceback.format_exc()))
+        logger(logging.CRITICAL, "Failed to schedule requests, error: %s" % (traceback.format_exc()))
 
 
-def __release_all_activities(stats, direction, rse_name, rse_id):
+def __release_all_activities(stats, direction, rse_name, rse_id, logger, session):
     """
     Release requests if activities should be ignored.
 
@@ -255,19 +252,19 @@ def __release_all_activities(stats, direction, rse_name, rse_id):
             if strategy == 'grouped_fifo':
                 deadline = stats.get('deadline')
                 volume = stats.get('volume')
-                release_waiting_requests_grouped_fifo(rse_id, count=to_be_released, direction=direction, volume=volume, deadline=deadline)
+                release_waiting_requests_grouped_fifo(rse_id, count=to_be_released, direction=direction, volume=volume, deadline=deadline, session=session)
             elif strategy == 'fifo':
-                release_waiting_requests_fifo(rse_id, count=to_be_released, direction=direction)
+                release_waiting_requests_fifo(rse_id, count=to_be_released, direction=direction, session=session)
         else:
-            logging.debug("Throttler has done nothing on rse %s (transfer > 0.8 * threshold)" % rse_name)
+            logger(logging.DEBUG, "Throttler has done nothing on rse %s (transfer > 0.8 * threshold)" % rse_name)
     elif waiting > 0 or not threshold:
-        logging.debug("Throttler remove limits(threshold: %s) and release all waiting requests, rse %s" % (threshold, rse_name))
-        delete_rse_transfer_limits(rse_id, activity='all_activities')
-        release_all_waiting_requests(rse_id, direction=direction)
+        logger(logging.DEBUG, "Throttler remove limits(threshold: %s) and release all waiting requests, rse %s" % (threshold, rse_name))
+        delete_rse_transfer_limits(rse_id, activity='all_activities', session=session)
+        release_all_waiting_requests(rse_id, direction=direction, session=session)
         record_counter('daemons.conveyor.throttler.delete_rse_transfer_limits.%s' % (rse_name))
 
 
-def __release_per_activity(stats, direction, rse_name, rse_id):
+def __release_per_activity(stats, direction, rse_name, rse_id, logger, session):
     """
     Release requests per activity.
 
@@ -281,16 +278,16 @@ def __release_per_activity(stats, direction, rse_name, rse_id):
         transfer = stats['activities'][activity]['transfer']
         waiting = stats['activities'][activity]['waiting']
         if waiting:
-            logging.debug("Request status for %s at %s: %s" % (activity, rse_name,
-                                                               stats['activities'][activity]))
+            logger(logging.DEBUG, "Request status for %s at %s: %s" % (activity, rse_name,
+                                                                       stats['activities'][activity]))
             if threshold is None:
-                logging.debug("Throttler remove limits(threshold: %s) and release all waiting requests for activity %s, rse_id %s" % (threshold, activity, rse_id))
-                delete_rse_transfer_limits(rse_id, activity=activity)
-                release_all_waiting_requests(rse_id, activity=activity, direction=direction)
+                logger(logging.DEBUG, "Throttler remove limits(threshold: %s) and release all waiting requests for activity %s, rse_id %s" % (threshold, activity, rse_id))
+                delete_rse_transfer_limits(rse_id, activity=activity, session=session)
+                release_all_waiting_requests(rse_id, activity=activity, direction=direction, session=session)
                 record_counter('daemons.conveyor.throttler.delete_rse_transfer_limits.%s.%s' % (activity, rse_name))
             elif transfer + waiting > threshold:
-                logging.debug("Throttler set limits for activity %s, rse %s" % (activity, rse_name))
-                set_rse_transfer_limits(rse_id, activity=activity, max_transfers=threshold, transfers=transfer, waitings=waiting)
+                logger(logging.DEBUG, "Throttler set limits for activity %s, rse %s" % (activity, rse_name))
+                set_rse_transfer_limits(rse_id, activity=activity, max_transfers=threshold, transfers=transfer, waitings=waiting, session=session)
                 record_gauge('daemons.conveyor.throttler.set_rse_transfer_limits.%s.%s.max_transfers' % (activity, rse_name), threshold)
                 record_gauge('daemons.conveyor.throttler.set_rse_transfer_limits.%s.%s.transfers' % (activity, rse_name), transfer)
                 record_gauge('daemons.conveyor.throttler.set_rse_transfer_limits.%s.%s.waitings' % (activity, rse_name), waiting)
@@ -305,31 +302,31 @@ def __release_per_activity(stats, direction, rse_name, rse_id):
                     accounts = stats['activities'][activity]['accounts']
                     for account in accounts:
                         if nr_accounts == 1:
-                            logging.debug("Throttler release %s waiting requests for activity %s, rse %s, account %s " % (to_release, activity, rse_name, account))
-                            release_waiting_requests_fifo(rse_id, activity=activity, account=account, count=to_release, direction=direction)
+                            logger(logging.DEBUG, "Throttler release %s waiting requests for activity %s, rse %s, account %s " % (to_release, activity, rse_name, account))
+                            release_waiting_requests_fifo(rse_id, activity=activity, account=account, count=to_release, direction=direction, session=session)
                             record_gauge('daemons.conveyor.throttler.release_waiting_requests.%s.%s.%s' % (activity, rse_name, account), to_release)
                         elif accounts[account]['transfer'] > threshold_per_account:
-                            logging.debug("Throttler will not release  %s waiting requests for activity %s, rse %s, account %s: It queued more transfers than its share " %
-                                          (accounts[account]['waiting'], activity, rse_name, account))
+                            logger(logging.DEBUG, "Throttler will not release  %s waiting requests for activity %s, rse %s, account %s: It queued more transfers than its share " %
+                                   (accounts[account]['waiting'], activity, rse_name, account))
                             nr_accounts -= 1
                             to_release_per_account = math.ceil(to_release / nr_accounts)
                         elif accounts[account]['waiting'] < to_release_per_account:
-                            logging.debug("Throttler release %s waiting requests for activity %s, rse %s, account %s " % (accounts[account]['waiting'], activity, rse_name, account))
-                            release_waiting_requests_fifo(rse_id, activity=activity, account=account, count=accounts[account]['waiting'], direction=direction)
+                            logger(logging.DEBUG, "Throttler release %s waiting requests for activity %s, rse %s, account %s " % (accounts[account]['waiting'], activity, rse_name, account))
+                            release_waiting_requests_fifo(rse_id, activity=activity, account=account, count=accounts[account]['waiting'], direction=direction, session=session)
                             record_gauge('daemons.conveyor.throttler.release_waiting_requests.%s.%s.%s' % (activity, rse_name, account), accounts[account]['waiting'])
                             to_release = to_release - accounts[account]['waiting']
                             nr_accounts -= 1
                             to_release_per_account = math.ceil(to_release / nr_accounts)
                         else:
-                            logging.debug("Throttler release %s waiting requests for activity %s, rse %s, account %s " % (to_release_per_account, activity, rse_name, account))
-                            release_waiting_requests_fifo(rse_id, activity=activity, account=account, count=to_release_per_account, direction=direction)
+                            logger(logging.DEBUG, "Throttler release %s waiting requests for activity %s, rse %s, account %s " % (to_release_per_account, activity, rse_name, account))
+                            release_waiting_requests_fifo(rse_id, activity=activity, account=account, count=to_release_per_account, direction=direction, session=session)
                             record_gauge('daemons.conveyor.throttler.release_waiting_requests.%s.%s.%s' % (activity, rse_name, account), to_release_per_account)
                             to_release = to_release - to_release_per_account
                             nr_accounts -= 1
                 else:
-                    logging.debug("Throttler has done nothing for activity %s on rse %s (transfer > 0.8 * threshold)" % (activity, rse_name))
+                    logger(logging.DEBUG, "Throttler has done nothing for activity %s on rse %s (transfer > 0.8 * threshold)" % (activity, rse_name))
             elif waiting > 0:
-                logging.debug("Throttler remove limits(threshold: %s) and release all waiting requests for activity %s, rse %s" % (threshold, activity, rse_name))
-                delete_rse_transfer_limits(rse_id, activity=activity)
-                release_all_waiting_requests(rse_id, activity=activity, direction=direction)
+                logger(logging.DEBUG, "Throttler remove limits(threshold: %s) and release all waiting requests for activity %s, rse %s" % (threshold, activity, rse_name))
+                delete_rse_transfer_limits(rse_id, activity=activity, session=session)
+                release_all_waiting_requests(rse_id, activity=activity, direction=direction, session=session)
                 record_counter('daemons.conveyor.throttler.delete_rse_transfer_limits.%s.%s' % (activity, rse_name))
