@@ -1,4 +1,5 @@
-# Copyright 2014-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2019-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,31 +14,25 @@
 # limitations under the License.
 #
 # Authors:
-# - Ruturaj Gujar, <ruturaj.gujar23@gmail.com>, 2019
-#
-# PY3K COMPATIBLE
+# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2021
 
 import logging
 import os
 import socket
 import threading
 import time
-import sys
 
+import rucio.db.sqla.util
+from rucio.common import exception
+from rucio.common.logging import setup_logging
 from rucio.common.utils import get_thread_with_periodic_running_function
-from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.did import create_reports
-from rucio.common.config import config_get
+from rucio.core.heartbeat import live, die, sanity_check
 
 graceful_stop = threading.Event()
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 
 def aggregate_events(once=False):
     """
@@ -78,6 +73,11 @@ def run(once=False, threads=1):
     """
     Starts up the follower threads
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise exception.DatabaseException('Database was not updated, daemon won\'t start')
+
     hostname = socket.gethostname()
     sanity_check(executable='rucio-follower', hostname=hostname)
 
