@@ -1,4 +1,5 @@
-# Copyright 2013-2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2013-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,10 +21,8 @@
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Brandon White <bjwhite@fnal.gov>, 2019
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
 
 """
 Judge-Evaluator is a daemon to re-evaluate and execute replication rules.
@@ -32,35 +31,27 @@ Judge-Evaluator is a daemon to re-evaluate and execute replication rules.
 import logging
 import os
 import socket
-import sys
 import threading
 import time
 import traceback
-
 from datetime import datetime, timedelta
-from re import match
 from random import randint
-from six import iteritems
+from re import match
 
+from six import iteritems
 from sqlalchemy.exc import DatabaseError
 from sqlalchemy.orm.exc import FlushError
 
-from rucio.common.config import config_get
+import rucio.db.sqla.util
 from rucio.common.exception import DatabaseException, DataIdentifierNotFound, ReplicationRuleCreationTemporaryFailed
+from rucio.common.logging import setup_logging
 from rucio.common.types import InternalScope
 from rucio.core.heartbeat import live, die, sanity_check
-from rucio.core.rule import re_evaluate_did, get_updated_dids, delete_updated_did
 from rucio.core.monitor import record_counter
+from rucio.core.rule import re_evaluate_did, get_updated_dids, delete_updated_did
 
 graceful_stop = threading.Event()
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 
 def re_evaluator(once=False):
     """
@@ -183,6 +174,10 @@ def run(once=False, threads=1):
     """
     Starts up the Judge-Eval threads.
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise DatabaseException('Database was not updated, daemon won\'t start')
 
     executable = 'judge-evaluator'
     hostname = socket.gethostname()
