@@ -1,4 +1,5 @@
-# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2013-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,13 +15,12 @@
 #
 # Authors:
 # - Martin Barisits <martin.barisits@cern.ch>, 2013-2016
-# - Vincent Garonne <vgaronne@gmail.com>, 2014-2018
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2018
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2015
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Brandon White <bjwhite@fnal.gov>, 2019-2020
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
+# - Brandon White <bjwhite@fnal.gov>, 2019
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 """
 Judge-Repairer is a daemon to repair stuck replication rules.
@@ -29,33 +29,26 @@ Judge-Repairer is a daemon to repair stuck replication rules.
 import logging
 import os
 import socket
-import sys
 import threading
 import time
 import traceback
-
 from copy import deepcopy
 from datetime import datetime, timedelta
-from re import match
 from random import randint
+from re import match
 
 from sqlalchemy.exc import DatabaseError
 
-from rucio.common.config import config_get
+import rucio.db.sqla.util
+from rucio.common import exception
 from rucio.common.exception import DatabaseException
+from rucio.common.logging import setup_logging
 from rucio.core.heartbeat import live, die, sanity_check
-from rucio.core.rule import repair_rule, get_stuck_rules
 from rucio.core.monitor import record_counter
+from rucio.core.rule import repair_rule, get_stuck_rules
 
 graceful_stop = threading.Event()
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 
 def rule_repairer(once=False):
     """
@@ -92,6 +85,7 @@ def rule_repairer(once=False):
                                     delta=-1 if once else 1800,
                                     limit=100,
                                     blacklisted_rules=[key for key in paused_rules])
+
             logging.debug('rule_repairer[%s/%s] index query time %f fetch size is %d' % (heartbeat['assign_thread'], heartbeat['nr_threads'], time.time() - start, len(rules)))
 
             if not rules and not once:
@@ -153,6 +147,10 @@ def run(once=False, threads=1):
     """
     Starts up the Judge-Repairer threads.
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise exception.DatabaseException('Database was not updated, daemon won\'t start')
 
     executable = 'judge-repairer'
     hostname = socket.gethostname()
