@@ -1,4 +1,5 @@
-# Copyright 2016-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2016-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,16 +14,17 @@
 # limitations under the License.
 #
 # Authors:
-# - Vincent Garonne <vgaronne@gmail.com>, 2016-2018
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2016-2018
 # - Martin Barisits <martin.barisits@cern.ch>, 2016
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2016-2019
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2016-2021
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2020
-# - Brandon White <bjwhite@fnal.gov>, 2019-2020
+# - Brandon White <bjwhite@fnal.gov>, 2019
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2020
+# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2020-2021
+# - Eric Vaandering <ewv@fnal.gov>, 2021
 
 '''
 Dark Reaper is a daemon to manage quarantined file deletion.
@@ -38,29 +40,24 @@ import threading
 import time
 import traceback
 
-from rucio.common.config import config_get, config_get_bool
+import rucio.db.sqla.util
+from rucio.common import exception
+from rucio.common.config import config_get_bool
 from rucio.common.exception import (SourceNotFound, DatabaseException, ServiceUnavailable,
                                     RSEAccessDenied, ResourceTemporaryUnavailable,
                                     RSENotFound, VONotFound)
+from rucio.common.logging import setup_logging
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.message import add_message
 from rucio.core.quarantined_replica import (list_quarantined_replicas,
                                             delete_quarantined_replicas,
                                             list_rses)
-from rucio.rse import rsemanager as rsemgr
 from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.vo import list_vos
-
+from rucio.rse import rsemanager as rsemgr
 
 logging.getLogger("requests").setLevel(logging.CRITICAL)
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 GRACEFUL_STOP = threading.Event()
 
 
@@ -81,14 +78,16 @@ def reaper(rses=[], worker_number=0, total_workers=1, chunk_size=100, once=False
     thread = threading.current_thread()
     hostname = socket.gethostname()
     executable = ' '.join(sys.argv)
-    hash_executable = hashlib.sha256(sys.argv[0] + ''.join(rses)).hexdigest()
+    hash_executable = hashlib.sha256((sys.argv[0] + ''.join(rses)).encode()).hexdigest()
     sanity_check(executable=None, hostname=hostname)
 
     while not GRACEFUL_STOP.is_set():
         try:
             # heartbeat
-            heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=thread, hash_executable=hash_executable)
-            logging.info('Dark Reaper({0[worker_number]}/{0[total_workers]}): Live gives {0[heartbeat]}'.format(locals()))
+            heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=thread,
+                             hash_executable=hash_executable)
+            logging.info('Dark Reaper({0[worker_number]}/{0[total_workers]}): Live gives {0[heartbeat]}'
+                         .format(locals()))
             nothing_to_do = True
 
             random.shuffle(rses)
@@ -109,14 +108,19 @@ def reaper(rses=[], worker_number=0, total_workers=1, chunk_size=100, once=False
                         if replica['scope']:
                             scope = replica['scope'].external
                         try:
-                            pfn = str(rsemgr.lfns2pfns(rse_settings=rse_info,
-                                                       lfns=[{'scope': scope, 'name': replica['name'], 'path': replica['path']}],
-                                                       operation='delete', scheme=scheme).values()[0])
-                            logging.info('Dark Reaper %s-%s: Deletion ATTEMPT of %s:%s as %s on %s', worker_number, total_workers, scope, replica['name'], pfn, rse)
+                            pfn = str(list(rsemgr.lfns2pfns(rse_settings=rse_info,
+                                                            lfns=[{'scope': scope,
+                                                                   'name': replica['name'],
+                                                                   'path': replica['path']}],
+                                                            operation='delete',
+                                                            scheme=scheme).values())[0])
+                            logging.info('Dark Reaper %s-%s: Deletion ATTEMPT of %s:%s as %s on %s',
+                                         worker_number, total_workers, scope, replica['name'], pfn, rse)
                             start = time.time()
                             prot.delete(pfn)
                             duration = time.time() - start
-                            logging.info('Dark Reaper %s-%s: Deletion SUCCESS of %s:%s as %s on %s in %s seconds', worker_number, total_workers, scope, replica['name'], pfn, rse, duration)
+                            logging.info('Dark Reaper %s-%s: Deletion SUCCESS of %s:%s as %s on %s in %s seconds',
+                                         worker_number, total_workers, scope, replica['name'], pfn, rse, duration)
                             payload = {'scope': scope,
                                        'name': replica['name'],
                                        'rse': rse,
@@ -131,11 +135,13 @@ def reaper(rses=[], worker_number=0, total_workers=1, chunk_size=100, once=False
                             add_message('deletion-done', payload)
                             deleted_replicas.append(replica)
                         except SourceNotFound:
-                            err_msg = 'Dark Reaper %s-%s: Deletion NOTFOUND of %s:%s as %s on %s' % (worker_number, total_workers, scope, replica['name'], pfn, rse)
+                            err_msg = ('Dark Reaper %s-%s: Deletion NOTFOUND of %s:%s as %s on %s'
+                                       % (worker_number, total_workers, scope, replica['name'], pfn, rse))
                             logging.warning(err_msg)
                             deleted_replicas.append(replica)
                         except (ServiceUnavailable, RSEAccessDenied, ResourceTemporaryUnavailable) as error:
-                            err_msg = 'Dark Reaper %s-%s: Deletion NOACCESS of %s:%s as %s on %s: %s' % (worker_number, total_workers, scope, replica['name'], pfn, rse, str(error))
+                            err_msg = ('Dark Reaper %s-%s: Deletion NOACCESS of %s:%s as %s on %s: %s'
+                                       % (worker_number, total_workers, scope, replica['name'], pfn, rse, str(error)))
                             logging.warning(err_msg)
                             payload = {'scope': scope,
                                        'name': replica['name'],
@@ -185,7 +191,7 @@ def stop(signum=None, frame=None):
     GRACEFUL_STOP.set()
 
 
-def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None, all_rses=False,
+def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None,
         exclude_rses=None, include_rses=None, vos=None, delay_seconds=0):
     """
     Starts up the reaper threads.
@@ -200,6 +206,11 @@ def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None, all_r
     :param vos: VOs on which to look for RSEs. Only used in multi-VO mode.
                 If None, we either use all VOs if run from "def", or the current VO otherwise.
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise exception.DatabaseException('Database was not updated, daemon won\'t start')
+
     logging.info('main: starting processes')
 
     multi_vo = config_get_bool('common', 'multi_vo', raise_exception=False, default=False)
@@ -207,49 +218,43 @@ def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None, all_r
         if vos:
             logging.warning('Ignoring argument vos, this is only applicable in a multi-VO setup.')
         vos = ['def']
-
-        # Preserve the old behaviour until a feature release
-        all_rses = list_rses()
     else:
         if vos:
             invalid = set(vos) - set([v['vo'] for v in list_vos()])
             if invalid:
-                msg = 'VO{} {} cannot be found'.format('s' if len(invalid) > 1 else '', ', '.join([repr(v) for v in invalid]))
+                msg = 'VO{} {} cannot be found'.format('s' if len(invalid) > 1 else '',
+                                                       ', '.join([repr(v) for v in invalid]))
                 raise VONotFound(msg)
         else:
             vos = [v['vo'] for v in list_vos()]
-        logging.info('Dark Reaper: This instance will work on VO%s: %s' % ('s' if len(vos) > 1 else '', ', '.join([v for v in vos])))
+        logging.info('Dark Reaper: This instance will work on VO%s: %s'
+                     % ('s' if len(vos) > 1 else '', ', '.join([v for v in vos])))
 
-        all_rses = []
-        for vo in vos:
-            vo_rses = list_rses(filters={'vo': vo})
-            if vo_rses:
-                all_rses.extend(vo_rses)
+    all_rses = []
+    for vo in vos:
+        all_rses.extend(list_rses(filters={'vo': vo}))
 
-    if all_rses:
-        rses = all_rses
+    if rses:
+        invalid = set(rses) - set([rse['rse'] for rse in all_rses])
+        if invalid:
+            msg = 'RSE{} {} cannot be found'.format('s' if len(invalid) > 1 else '',
+                                                    ', '.join([repr(rse) for rse in invalid]))
+            raise RSENotFound(msg)
+        rses = [rse for rse in all_rses if rse['rse'] in rses]
     else:
-        if rses:
-            invalid = set(rses) - set([rse['rse'] for rse in all_rses])
-            if invalid:
-                msg = 'RSE{} {} cannot be found'.format('s' if len(invalid) > 1 else '',
-                                                        ', '.join([repr(rse) for rse in invalid]))
-                raise RSENotFound(msg)
-            rses = [rse for rse in all_rses if rse['rse'] in rses]
-        else:
-            rses = all_rses
+        rses = all_rses
 
-        if exclude_rses:
-            excluded_rses = parse_expression(exclude_rses)
-            rses = [rse for rse in rses if rse not in excluded_rses]
+    if exclude_rses:
+        excluded_rses = [rse['id'] for rse in parse_expression(exclude_rses)]
+        rses = [rse for rse in rses if rse not in excluded_rses]
 
-        if include_rses:
-            included_rses = parse_expression(include_rses)
-            rses = [rse for rse in rses if rse in included_rses]
+    if include_rses:
+        included_rses = [rse['id'] for rse in parse_expression(include_rses)]
+        rses = [rse for rse in rses if rse in included_rses]
 
-        if not rses:
-            logging.error('Dark Reaper: No RSEs found. Exiting.')
-            return
+    if not rses:
+        logging.error('Dark Reaper: No RSEs found. Exiting.')
+        return
 
     threads = []
     for worker in range(total_workers):
@@ -259,7 +264,8 @@ def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None, all_r
                   'once': once,
                   'chunk_size': chunk_size,
                   'scheme': scheme}
-        threads.append(threading.Thread(target=reaper, kwargs=kwargs, name='Worker: %s, Total_Workers: %s' % (worker, total_workers)))
+        threads.append(threading.Thread(target=reaper, kwargs=kwargs,
+                                        name='Worker: %s, Total_Workers: %s' % (worker, total_workers)))
     [t.start() for t in threads]
     while threads[0].is_alive():
         [t.join(timeout=3.14) for t in threads]
