@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014-2020 CERN
+# Copyright 2014-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,27 +16,24 @@
 # Authors:
 # - Martin Barisits <martin.barisits@cern.ch>, 2014
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2017
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2018
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2018-2021
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2018-2021
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Muhammad Aditya Hilmy <didithilmy@gmail.com>, 2020
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-
-from json import loads
-import logging
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 
 from flask import Flask, Blueprint, request
-from flask.views import MethodView
 
-from rucio.api.account_limit import set_local_account_limit, delete_local_account_limit, set_global_account_limit, delete_global_account_limit
+from rucio.api.account_limit import set_local_account_limit, delete_local_account_limit, set_global_account_limit, \
+    delete_global_account_limit
 from rucio.common.exception import RSENotFound, AccessDenied, AccountNotFound
-from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers
-from rucio.web.rest.utils import generate_http_error_flask
+from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, ErrorHandlingMethodView, \
+    generate_http_error_flask, json_parameters, param_get
 
 
-class LocalAccountLimit(MethodView):
+class LocalAccountLimit(ErrorHandlingMethodView):
     def post(self, account, rse):
         """ Create or update an account limit.
 
@@ -49,31 +46,14 @@ class LocalAccountLimit(MethodView):
         :status 404: RSE not found.
         :status 404: Account not found
         """
-        json_data = request.data
+        parameters = json_parameters()
+        bytes_param = param_get(parameters, 'bytes')
         try:
-            parameter = loads(json_data)
-        except ValueError:
-            return generate_http_error_flask(400, 'ValueError', 'cannot decode json parameter dictionary')
-        try:
-            bytes = parameter['bytes']
-        except KeyError as error:
-            if error.args[0] == 'type':
-                return generate_http_error_flask(400, 'KeyError', '%s not defined' % str(error))
-            raise
-        except TypeError:
-            return generate_http_error_flask(400, 'TypeError', 'body must be a json dictionary')
-
-        try:
-            set_local_account_limit(account=account, rse=rse, bytes=bytes, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
+            set_local_account_limit(account=account, rse=rse, bytes=bytes_param, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
         except AccessDenied as error:
-            return generate_http_error_flask(401, 'AccessDenied', error.args[0])
-        except RSENotFound as error:
-            return generate_http_error_flask(404, 'RSENotFound', error.args[0])
-        except AccountNotFound as error:
-            return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
-        except Exception as error:
-            logging.exception("Internal Error")
-            return str(error), 500
+            return generate_http_error_flask(401, error)
+        except (RSENotFound, AccountNotFound) as error:
+            return generate_http_error_flask(404, error)
 
         return 'Created', 201
 
@@ -92,18 +72,14 @@ class LocalAccountLimit(MethodView):
         try:
             delete_local_account_limit(account=account, rse=rse, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
         except AccessDenied as error:
-            return generate_http_error_flask(401, 'AccessDenied', error.args[0])
-        except AccountNotFound as error:
-            return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
-        except RSENotFound as error:
-            return generate_http_error_flask(404, 'RSENotFound', error.args[0])
-        except Exception as error:
-            logging.exception("Internal Error")
-            return str(error), 500
+            return generate_http_error_flask(401, error)
+        except (AccountNotFound, RSENotFound) as error:
+            return generate_http_error_flask(404, error)
+
         return '', 200
 
 
-class GlobalAccountLimit(MethodView):
+class GlobalAccountLimit(ErrorHandlingMethodView):
     def post(self, account, rse_expression):
         """ Create or update an account limit.
 
@@ -116,31 +92,20 @@ class GlobalAccountLimit(MethodView):
         :status 404: RSE not found.
         :status 404: Account not found
         """
-        json_data = request.data
+        parameters = json_parameters()
+        bytes_param = param_get(parameters, 'bytes')
         try:
-            parameter = loads(json_data)
-        except ValueError:
-            return generate_http_error_flask(400, 'ValueError', 'cannot decode json parameter dictionary')
-        try:
-            bytes = parameter['bytes']
-        except KeyError as error:
-            if error.args[0] == 'type':
-                return generate_http_error_flask(400, 'KeyError', '%s not defined' % str(error))
-            raise
-        except TypeError:
-            return generate_http_error_flask(400, 'TypeError', 'body must be a json dictionary')
-
-        try:
-            set_global_account_limit(account=account, rse_expression=rse_expression, bytes=bytes, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
+            set_global_account_limit(
+                account=account,
+                rse_expression=rse_expression,
+                bytes=bytes_param,
+                issuer=request.environ.get('issuer'),
+                vo=request.environ.get('vo'),
+            )
         except AccessDenied as error:
-            return generate_http_error_flask(401, 'AccessDenied', error.args[0])
-        except RSENotFound as error:
-            return generate_http_error_flask(404, 'RSENotFound', error.args[0])
-        except AccountNotFound as error:
-            return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
-        except Exception as error:
-            logging.exception("Internal Error")
-            return str(error), 500
+            return generate_http_error_flask(401, error)
+        except (RSENotFound, AccountNotFound) as error:
+            return generate_http_error_flask(404, error)
 
         return 'Created', 201
 
@@ -159,14 +124,10 @@ class GlobalAccountLimit(MethodView):
         try:
             delete_global_account_limit(account=account, rse_expression=rse_expression, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
         except AccessDenied as error:
-            return generate_http_error_flask(401, 'AccessDenied', error.args[0])
-        except AccountNotFound as error:
-            return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
-        except RSENotFound as error:
-            return generate_http_error_flask(404, 'RSENotFound', error.args[0])
-        except Exception as error:
-            logging.exception("Internal Error")
-            return str(error), 500
+            return generate_http_error_flask(401, error)
+        except (AccountNotFound, RSENotFound) as error:
+            return generate_http_error_flask(404, error)
+
         return '', 200
 
 
